@@ -4,18 +4,18 @@ from __future__ import annotations as _annotations
 
 import dataclasses
 import sys
+import warnings
 from functools import partialmethod
 from types import FunctionType
-from typing import TYPE_CHECKING, Any, Callable, TypeVar, Union, cast, overload
+from typing import TYPE_CHECKING, Annotated, Any, Callable, Literal, TypeVar, Union, cast, overload
 
-from pydantic_core import core_schema
-from pydantic_core import core_schema as _core_schema
-from typing_extensions import Annotated, Literal, TypeAlias
+from pydantic_core import PydanticUndefined, core_schema
+from typing_extensions import Self, TypeAlias
 
-from . import GetCoreSchemaHandler as _GetCoreSchemaHandler
-from ._internal import _core_metadata, _decorators, _generics, _internal_dataclass
+from ._internal import _decorators, _generics, _internal_dataclass
 from .annotated_handlers import GetCoreSchemaHandler
 from .errors import PydanticUserError
+from .warnings import ArbitraryTypeWarning
 
 if sys.version_info < (3, 11):
     from typing_extensions import Protocol
@@ -27,7 +27,8 @@ _inspect_validator = _decorators.inspect_validator
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class AfterValidator:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *after* validators](../concepts/validators.md#field-after-validator)
 
     A metadata class that indicates that a validation should be applied **after** the inner validation logic.
 
@@ -35,8 +36,8 @@ class AfterValidator:
         func: The validator function.
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated
 
         from pydantic import AfterValidator, BaseModel, ValidationError
 
@@ -70,29 +71,36 @@ class AfterValidator:
 
     func: core_schema.NoInfoValidatorFunction | core_schema.WithInfoValidatorFunction
 
-    def __get_pydantic_core_schema__(self, source_type: Any, handler: _GetCoreSchemaHandler) -> core_schema.CoreSchema:
+    def __get_pydantic_core_schema__(self, source_type: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
         schema = handler(source_type)
-        info_arg = _inspect_validator(self.func, 'after')
+        info_arg = _inspect_validator(self.func, mode='after', type='field')
         if info_arg:
             func = cast(core_schema.WithInfoValidatorFunction, self.func)
-            return core_schema.with_info_after_validator_function(func, schema=schema, field_name=handler.field_name)
+            return core_schema.with_info_after_validator_function(func, schema=schema)
         else:
             func = cast(core_schema.NoInfoValidatorFunction, self.func)
             return core_schema.no_info_after_validator_function(func, schema=schema)
 
+    @classmethod
+    def _from_decorator(cls, decorator: _decorators.Decorator[_decorators.FieldValidatorDecoratorInfo]) -> Self:
+        return cls(func=decorator.func)
+
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class BeforeValidator:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *before* validators](../concepts/validators.md#field-before-validator)
 
     A metadata class that indicates that a validation should be applied **before** the inner validation logic.
 
     Attributes:
         func: The validator function.
+        json_schema_input_type: The input type used to generate the appropriate
+            JSON Schema (in validation mode). The actual input type is `Any`.
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated
 
         from pydantic import BaseModel, BeforeValidator
 
@@ -113,46 +121,90 @@ class BeforeValidator:
     """
 
     func: core_schema.NoInfoValidatorFunction | core_schema.WithInfoValidatorFunction
+    json_schema_input_type: Any = PydanticUndefined
 
-    def __get_pydantic_core_schema__(self, source_type: Any, handler: _GetCoreSchemaHandler) -> core_schema.CoreSchema:
+    def __get_pydantic_core_schema__(self, source_type: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
         schema = handler(source_type)
-        info_arg = _inspect_validator(self.func, 'before')
+        input_schema = (
+            None
+            if self.json_schema_input_type is PydanticUndefined
+            else handler.generate_schema(self.json_schema_input_type)
+        )
+
+        info_arg = _inspect_validator(self.func, mode='before', type='field')
         if info_arg:
             func = cast(core_schema.WithInfoValidatorFunction, self.func)
-            return core_schema.with_info_before_validator_function(func, schema=schema, field_name=handler.field_name)
+            return core_schema.with_info_before_validator_function(
+                func,
+                schema=schema,
+                json_schema_input_schema=input_schema,
+            )
         else:
             func = cast(core_schema.NoInfoValidatorFunction, self.func)
-            return core_schema.no_info_before_validator_function(func, schema=schema)
+            return core_schema.no_info_before_validator_function(
+                func, schema=schema, json_schema_input_schema=input_schema
+            )
+
+    @classmethod
+    def _from_decorator(cls, decorator: _decorators.Decorator[_decorators.FieldValidatorDecoratorInfo]) -> Self:
+        return cls(
+            func=decorator.func,
+            json_schema_input_type=decorator.info.json_schema_input_type,
+        )
 
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class PlainValidator:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *plain* validators](../concepts/validators.md#field-plain-validator)
 
     A metadata class that indicates that a validation should be applied **instead** of the inner validation logic.
 
+    !!! note
+        Before v2.9, `PlainValidator` wasn't always compatible with JSON Schema generation for `mode='validation'`.
+        You can now use the `json_schema_input_type` argument to specify the input type of the function
+        to be used in the JSON schema when `mode='validation'` (the default). See the example below for more details.
+
     Attributes:
         func: The validator function.
+        json_schema_input_type: The input type used to generate the appropriate
+            JSON Schema (in validation mode). The actual input type is `Any`.
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated, Union
 
         from pydantic import BaseModel, PlainValidator
 
-        MyInt = Annotated[int, PlainValidator(lambda v: int(v) + 1)]
+        def validate(v: object) -> int:
+            if not isinstance(v, (int, str)):
+                raise ValueError(f'Expected int or str, go {type(v)}')
+
+            return int(v) + 1
+
+        MyInt = Annotated[
+            int,
+            PlainValidator(validate, json_schema_input_type=Union[str, int]),  # (1)!
+        ]
 
         class Model(BaseModel):
             a: MyInt
 
         print(Model(a='1').a)
         #> 2
+
+        print(Model(a=1).a)
+        #> 2
         ```
+
+        1. In this example, we've specified the `json_schema_input_type` as `Union[str, int]` which indicates to the JSON schema
+        generator that in validation mode, the input type for the `a` field can be either a [`str`][] or an [`int`][].
     """
 
     func: core_schema.NoInfoValidatorFunction | core_schema.WithInfoValidatorFunction
+    json_schema_input_type: Any = Any
 
-    def __get_pydantic_core_schema__(self, source_type: Any, handler: _GetCoreSchemaHandler) -> core_schema.CoreSchema:
+    def __get_pydantic_core_schema__(self, source_type: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
         # Note that for some valid uses of PlainValidator, it is not possible to generate a core schema for the
         # source_type, so calling `handler(source_type)` will error, which prevents us from generating a proper
         # serialization schema. To work around this for use cases that will not involve serialization, we simply
@@ -162,34 +214,60 @@ class PlainValidator:
 
         try:
             schema = handler(source_type)
-            serialization = core_schema.wrap_serializer_function_ser_schema(function=lambda v, h: h(v), schema=schema)
+            # TODO if `schema['serialization']` is one of `'include-exclude-dict/sequence',
+            # schema validation will fail. That's why we use 'type ignore' comments below.
+            serialization = schema.get(
+                'serialization',
+                core_schema.wrap_serializer_function_ser_schema(
+                    function=lambda v, h: h(v),
+                    schema=schema,
+                    return_schema=handler.generate_schema(source_type),
+                ),
+            )
         except PydanticSchemaGenerationError:
             serialization = None
 
-        info_arg = _inspect_validator(self.func, 'plain')
+        input_schema = handler.generate_schema(self.json_schema_input_type)
+
+        info_arg = _inspect_validator(self.func, mode='plain', type='field')
         if info_arg:
             func = cast(core_schema.WithInfoValidatorFunction, self.func)
             return core_schema.with_info_plain_validator_function(
-                func, field_name=handler.field_name, serialization=serialization
+                func,
+                serialization=serialization,  # pyright: ignore[reportArgumentType]
+                json_schema_input_schema=input_schema,
             )
         else:
             func = cast(core_schema.NoInfoValidatorFunction, self.func)
-            return core_schema.no_info_plain_validator_function(func, serialization=serialization)
+            return core_schema.no_info_plain_validator_function(
+                func,
+                serialization=serialization,  # pyright: ignore[reportArgumentType]
+                json_schema_input_schema=input_schema,
+            )
+
+    @classmethod
+    def _from_decorator(cls, decorator: _decorators.Decorator[_decorators.FieldValidatorDecoratorInfo]) -> Self:
+        return cls(
+            func=decorator.func,
+            json_schema_input_type=decorator.info.json_schema_input_type,
+        )
 
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class WrapValidator:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *wrap* validators](../concepts/validators.md#field-wrap-validator)
 
     A metadata class that indicates that a validation should be applied **around** the inner validation logic.
 
     Attributes:
         func: The validator function.
+        json_schema_input_type: The input type used to generate the appropriate
+            JSON Schema (in validation mode). The actual input type is `Any`.
 
-    ```py
+    ```python
     from datetime import datetime
-
-    from typing_extensions import Annotated
+    from typing import Annotated
 
     from pydantic import BaseModel, ValidationError, WrapValidator
 
@@ -216,70 +294,84 @@ class WrapValidator:
     """
 
     func: core_schema.NoInfoWrapValidatorFunction | core_schema.WithInfoWrapValidatorFunction
+    json_schema_input_type: Any = PydanticUndefined
 
-    def __get_pydantic_core_schema__(self, source_type: Any, handler: _GetCoreSchemaHandler) -> core_schema.CoreSchema:
+    def __get_pydantic_core_schema__(self, source_type: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
         schema = handler(source_type)
-        info_arg = _inspect_validator(self.func, 'wrap')
+        input_schema = (
+            None
+            if self.json_schema_input_type is PydanticUndefined
+            else handler.generate_schema(self.json_schema_input_type)
+        )
+
+        info_arg = _inspect_validator(self.func, mode='wrap', type='field')
         if info_arg:
             func = cast(core_schema.WithInfoWrapValidatorFunction, self.func)
-            return core_schema.with_info_wrap_validator_function(func, schema=schema, field_name=handler.field_name)
+            return core_schema.with_info_wrap_validator_function(
+                func,
+                schema=schema,
+                json_schema_input_schema=input_schema,
+            )
         else:
             func = cast(core_schema.NoInfoWrapValidatorFunction, self.func)
-            return core_schema.no_info_wrap_validator_function(func, schema=schema)
+            return core_schema.no_info_wrap_validator_function(
+                func,
+                schema=schema,
+                json_schema_input_schema=input_schema,
+            )
+
+    @classmethod
+    def _from_decorator(cls, decorator: _decorators.Decorator[_decorators.FieldValidatorDecoratorInfo]) -> Self:
+        return cls(
+            func=decorator.func,
+            json_schema_input_type=decorator.info.json_schema_input_type,
+        )
 
 
 if TYPE_CHECKING:
 
     class _OnlyValueValidatorClsMethod(Protocol):
-        def __call__(self, cls: Any, value: Any, /) -> Any:
-            ...
+        def __call__(self, cls: Any, value: Any, /) -> Any: ...
 
     class _V2ValidatorClsMethod(Protocol):
-        def __call__(self, cls: Any, value: Any, info: _core_schema.ValidationInfo, /) -> Any:
-            ...
+        def __call__(self, cls: Any, value: Any, info: core_schema.ValidationInfo[Any], /) -> Any: ...
+
+    class _OnlyValueWrapValidatorClsMethod(Protocol):
+        def __call__(self, cls: Any, value: Any, handler: core_schema.ValidatorFunctionWrapHandler, /) -> Any: ...
 
     class _V2WrapValidatorClsMethod(Protocol):
         def __call__(
             self,
             cls: Any,
             value: Any,
-            handler: _core_schema.ValidatorFunctionWrapHandler,
-            info: _core_schema.ValidationInfo,
+            handler: core_schema.ValidatorFunctionWrapHandler,
+            info: core_schema.ValidationInfo[Any],
             /,
-        ) -> Any:
-            ...
+        ) -> Any: ...
 
     _V2Validator = Union[
         _V2ValidatorClsMethod,
-        _core_schema.WithInfoValidatorFunction,
+        core_schema.WithInfoValidatorFunction,
         _OnlyValueValidatorClsMethod,
-        _core_schema.NoInfoValidatorFunction,
+        core_schema.NoInfoValidatorFunction,
     ]
 
     _V2WrapValidator = Union[
         _V2WrapValidatorClsMethod,
-        _core_schema.WithInfoWrapValidatorFunction,
+        core_schema.WithInfoWrapValidatorFunction,
+        _OnlyValueWrapValidatorClsMethod,
+        core_schema.NoInfoWrapValidatorFunction,
     ]
 
     _PartialClsOrStaticMethod: TypeAlias = Union[classmethod[Any, Any, Any], staticmethod[Any, Any], partialmethod[Any]]
 
     _V2BeforeAfterOrPlainValidatorType = TypeVar(
         '_V2BeforeAfterOrPlainValidatorType',
-        _V2Validator,
-        _PartialClsOrStaticMethod,
+        bound=Union[_V2Validator, _PartialClsOrStaticMethod],
     )
-    _V2WrapValidatorType = TypeVar('_V2WrapValidatorType', _V2WrapValidator, _PartialClsOrStaticMethod)
+    _V2WrapValidatorType = TypeVar('_V2WrapValidatorType', bound=Union[_V2WrapValidator, _PartialClsOrStaticMethod])
 
-
-@overload
-def field_validator(
-    field: str,
-    /,
-    *fields: str,
-    mode: Literal['before', 'after', 'plain'] = ...,
-    check_fields: bool | None = ...,
-) -> Callable[[_V2BeforeAfterOrPlainValidatorType], _V2BeforeAfterOrPlainValidatorType]:
-    ...
+FieldValidatorModes: TypeAlias = Literal['before', 'after', 'wrap', 'plain']
 
 
 @overload
@@ -289,11 +381,29 @@ def field_validator(
     *fields: str,
     mode: Literal['wrap'],
     check_fields: bool | None = ...,
-) -> Callable[[_V2WrapValidatorType], _V2WrapValidatorType]:
-    ...
+    json_schema_input_type: Any = ...,
+) -> Callable[[_V2WrapValidatorType], _V2WrapValidatorType]: ...
 
 
-FieldValidatorModes: TypeAlias = Literal['before', 'after', 'wrap', 'plain']
+@overload
+def field_validator(
+    field: str,
+    /,
+    *fields: str,
+    mode: Literal['before', 'plain'],
+    check_fields: bool | None = ...,
+    json_schema_input_type: Any = ...,
+) -> Callable[[_V2BeforeAfterOrPlainValidatorType], _V2BeforeAfterOrPlainValidatorType]: ...
+
+
+@overload
+def field_validator(
+    field: str,
+    /,
+    *fields: str,
+    mode: Literal['after'] = ...,
+    check_fields: bool | None = ...,
+) -> Callable[[_V2BeforeAfterOrPlainValidatorType], _V2BeforeAfterOrPlainValidatorType]: ...
 
 
 def field_validator(
@@ -302,13 +412,15 @@ def field_validator(
     *fields: str,
     mode: FieldValidatorModes = 'after',
     check_fields: bool | None = None,
+    json_schema_input_type: Any = PydanticUndefined,
 ) -> Callable[[Any], Any]:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#field-validators
+    """!!! abstract "Usage Documentation"
+        [field validators](../concepts/validators.md#field-validators)
 
     Decorate methods on the class indicating that they should be used to validate fields.
 
     Example usage:
-    ```py
+    ```python
     from typing import Any
 
     from pydantic import (
@@ -349,6 +461,9 @@ def field_validator(
         *fields: Additional field(s) the `field_validator` should be called on.
         mode: Specifies whether to validate the fields before or after validation.
         check_fields: Whether to check that the fields actually exist on the model.
+        json_schema_input_type: The input type of the function. This is only used to generate
+            the appropriate JSON Schema (in validation mode) and can only specified
+            when `mode` is either `'before'`, `'plain'` or `'wrap'`.
 
     Returns:
         A decorator that can be used to decorate a function to be used as a field_validator.
@@ -365,6 +480,16 @@ def field_validator(
             "E.g. usage should be `@validator('<field_name>', ...)`",
             code='validator-no-fields',
         )
+
+    if mode not in ('before', 'plain', 'wrap') and json_schema_input_type is not PydanticUndefined:
+        raise PydanticUserError(
+            f"`json_schema_input_type` can't be used when mode is set to {mode!r}",
+            code='validator-input-type',
+        )
+
+    if json_schema_input_type is PydanticUndefined and mode == 'plain':
+        json_schema_input_type = Any
+
     fields = field, *fields
     if not all(isinstance(field, str) for field in fields):
         raise PydanticUserError(
@@ -384,7 +509,9 @@ def field_validator(
         # auto apply the @classmethod decorator
         f = _decorators.ensure_classmethod_based_on_signature(f)
 
-        dec_info = _decorators.FieldValidatorDecoratorInfo(fields=fields, mode=mode, check_fields=check_fields)
+        dec_info = _decorators.FieldValidatorDecoratorInfo(
+            fields=fields, mode=mode, check_fields=check_fields, json_schema_input_type=json_schema_input_type
+        )
         return _decorators.PydanticDescriptorProxy(f, dec_info)
 
     return dec
@@ -394,8 +521,8 @@ _ModelType = TypeVar('_ModelType')
 _ModelTypeCo = TypeVar('_ModelTypeCo', covariant=True)
 
 
-class ModelWrapValidatorHandler(_core_schema.ValidatorFunctionWrapHandler, Protocol[_ModelTypeCo]):
-    """@model_validator decorated function handler argument type. This is used when `mode='wrap'`."""
+class ModelWrapValidatorHandler(core_schema.ValidatorFunctionWrapHandler, Protocol[_ModelTypeCo]):
+    """`@model_validator` decorated function handler argument type. This is used when `mode='wrap'`."""
 
     def __call__(  # noqa: D102
         self,
@@ -407,7 +534,7 @@ class ModelWrapValidatorHandler(_core_schema.ValidatorFunctionWrapHandler, Proto
 
 
 class ModelWrapValidatorWithoutInfo(Protocol[_ModelType]):
-    """A @model_validator decorated function signature.
+    """A `@model_validator` decorated function signature.
     This is used when `mode='wrap'` and the function does not have info argument.
     """
 
@@ -420,12 +547,11 @@ class ModelWrapValidatorWithoutInfo(Protocol[_ModelType]):
         value: Any,
         handler: ModelWrapValidatorHandler[_ModelType],
         /,
-    ) -> _ModelType:
-        ...
+    ) -> _ModelType: ...
 
 
 class ModelWrapValidator(Protocol[_ModelType]):
-    """A @model_validator decorated function signature. This is used when `mode='wrap'`."""
+    """A `@model_validator` decorated function signature. This is used when `mode='wrap'`."""
 
     def __call__(  # noqa: D102
         self,
@@ -435,14 +561,13 @@ class ModelWrapValidator(Protocol[_ModelType]):
         # thus validators _must_ handle all cases
         value: Any,
         handler: ModelWrapValidatorHandler[_ModelType],
-        info: _core_schema.ValidationInfo,
+        info: core_schema.ValidationInfo,
         /,
-    ) -> _ModelType:
-        ...
+    ) -> _ModelType: ...
 
 
 class FreeModelBeforeValidatorWithoutInfo(Protocol):
-    """A @model_validator decorated function signature.
+    """A `@model_validator` decorated function signature.
     This is used when `mode='before'` and the function does not have info argument.
     """
 
@@ -453,12 +578,11 @@ class FreeModelBeforeValidatorWithoutInfo(Protocol):
         # thus validators _must_ handle all cases
         value: Any,
         /,
-    ) -> Any:
-        ...
+    ) -> Any: ...
 
 
 class ModelBeforeValidatorWithoutInfo(Protocol):
-    """A @model_validator decorated function signature.
+    """A `@model_validator` decorated function signature.
     This is used when `mode='before'` and the function does not have info argument.
     """
 
@@ -470,8 +594,7 @@ class ModelBeforeValidatorWithoutInfo(Protocol):
         # thus validators _must_ handle all cases
         value: Any,
         /,
-    ) -> Any:
-        ...
+    ) -> Any: ...
 
 
 class FreeModelBeforeValidator(Protocol):
@@ -483,10 +606,9 @@ class FreeModelBeforeValidator(Protocol):
         # or anything else that gets passed to validate_python
         # thus validators _must_ handle all cases
         value: Any,
-        info: _core_schema.ValidationInfo,
+        info: core_schema.ValidationInfo[Any],
         /,
-    ) -> Any:
-        ...
+    ) -> Any: ...
 
 
 class ModelBeforeValidator(Protocol):
@@ -499,10 +621,9 @@ class ModelBeforeValidator(Protocol):
         # or anything else that gets passed to validate_python
         # thus validators _must_ handle all cases
         value: Any,
-        info: _core_schema.ValidationInfo,
+        info: core_schema.ValidationInfo[Any],
         /,
-    ) -> Any:
-        ...
+    ) -> Any: ...
 
 
 ModelAfterValidatorWithoutInfo = Callable[[_ModelType], _ModelType]
@@ -510,11 +631,11 @@ ModelAfterValidatorWithoutInfo = Callable[[_ModelType], _ModelType]
 have info argument.
 """
 
-ModelAfterValidator = Callable[[_ModelType, _core_schema.ValidationInfo], _ModelType]
+ModelAfterValidator = Callable[[_ModelType, core_schema.ValidationInfo[Any]], _ModelType]
 """A `@model_validator` decorated function signature. This is used when `mode='after'`."""
 
 _AnyModelWrapValidator = Union[ModelWrapValidator[_ModelType], ModelWrapValidatorWithoutInfo[_ModelType]]
-_AnyModeBeforeValidator = Union[
+_AnyModelBeforeValidator = Union[
     FreeModelBeforeValidator, ModelBeforeValidator, FreeModelBeforeValidatorWithoutInfo, ModelBeforeValidatorWithoutInfo
 ]
 _AnyModelAfterValidator = Union[ModelAfterValidator[_ModelType], ModelAfterValidatorWithoutInfo[_ModelType]]
@@ -526,16 +647,16 @@ def model_validator(
     mode: Literal['wrap'],
 ) -> Callable[
     [_AnyModelWrapValidator[_ModelType]], _decorators.PydanticDescriptorProxy[_decorators.ModelValidatorDecoratorInfo]
-]:
-    ...
+]: ...
 
 
 @overload
 def model_validator(
     *,
     mode: Literal['before'],
-) -> Callable[[_AnyModeBeforeValidator], _decorators.PydanticDescriptorProxy[_decorators.ModelValidatorDecoratorInfo]]:
-    ...
+) -> Callable[
+    [_AnyModelBeforeValidator], _decorators.PydanticDescriptorProxy[_decorators.ModelValidatorDecoratorInfo]
+]: ...
 
 
 @overload
@@ -544,20 +665,20 @@ def model_validator(
     mode: Literal['after'],
 ) -> Callable[
     [_AnyModelAfterValidator[_ModelType]], _decorators.PydanticDescriptorProxy[_decorators.ModelValidatorDecoratorInfo]
-]:
-    ...
+]: ...
 
 
 def model_validator(
     *,
     mode: Literal['wrap', 'before', 'after'],
 ) -> Any:
-    """Usage docs: https://docs.pydantic.dev/2.7/concepts/validators/#model-validators
+    """!!! abstract "Usage Documentation"
+        [Model Validators](../concepts/validators.md#model-validators)
 
     Decorate model methods for validation purposes.
 
     Example usage:
-    ```py
+    ```python
     from typing_extensions import Self
 
     from pydantic import BaseModel, ValidationError, model_validator
@@ -597,8 +718,9 @@ def model_validator(
     """
 
     def dec(f: Any) -> _decorators.PydanticDescriptorProxy[Any]:
-        # auto apply the @classmethod decorator
-        f = _decorators.ensure_classmethod_based_on_signature(f)
+        # auto apply the @classmethod decorator (except for *after* validators, which should be instance methods):
+        if mode != 'after':
+            f = _decorators.ensure_classmethod_based_on_signature(f)
         dec_info = _decorators.ModelValidatorDecoratorInfo(mode=mode)
         return _decorators.PydanticDescriptorProxy(f, dec_info)
 
@@ -619,7 +741,7 @@ else:
         '''Generic type for annotating a type that is an instance of a given class.
 
         Example:
-            ```py
+            ```python
             from pydantic import BaseModel, InstanceOf
 
             class Foo:
@@ -697,8 +819,10 @@ else:
 
         @classmethod
         def __get_pydantic_core_schema__(cls, source: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
-            original_schema = handler(source)
-            metadata = _core_metadata.build_metadata_dict(js_annotation_functions=[lambda _c, h: h(original_schema)])
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore', ArbitraryTypeWarning)
+                original_schema = handler(source)
+            metadata = {'pydantic_js_annotation_functions': [lambda _c, h: h(original_schema)]}
             return core_schema.any_schema(
                 metadata=metadata,
                 serialization=core_schema.wrap_serializer_function_ser_schema(
@@ -707,3 +831,53 @@ else:
             )
 
         __hash__ = object.__hash__
+
+
+_FromTypeT = TypeVar('_FromTypeT')
+
+
+class ValidateAs:
+    """A helper class to validate a custom type from a type that is natively supported by Pydantic.
+
+    Args:
+        from_type: The type natively supported by Pydantic to use to perform validation.
+        instantiation_hook: A callable taking the validated type as an argument, and returning
+            the populated custom type.
+
+    Example:
+        ```python {lint="skip"}
+        from typing import Annotated
+
+        from pydantic import BaseModel, TypeAdapter, ValidateAs
+
+        class MyCls:
+            def __init__(self, a: int) -> None:
+                self.a = a
+
+            def __repr__(self) -> str:
+                return f"MyCls(a={self.a})"
+
+        class Model(BaseModel):
+            a: int
+
+
+        ta = TypeAdapter(
+            Annotated[MyCls, ValidateAs(Model, lambda v: MyCls(a=v.a))]
+        )
+
+        print(ta.validate_python({'a': 1}))
+        #> MyCls(a=1)
+        ```
+    """
+
+    # TODO: make use of PEP 747
+    def __init__(self, from_type: type[_FromTypeT], /, instantiation_hook: Callable[[_FromTypeT], Any]) -> None:
+        self.from_type = from_type
+        self.instantiation_hook = instantiation_hook
+
+    def __get_pydantic_core_schema__(self, source: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
+        schema = handler(self.from_type)
+        return core_schema.no_info_after_validator_function(
+            self.instantiation_hook,
+            schema=schema,
+        )
